/*
 * General DV muxer/demuxer
 * Copyright (c) 2003 Roman Shaposhnik
 *
 * Many thanks to Dan Dennedy <dan@dennedy.org> for providing wealth
 * of DV technical info.
 *
 * Raw DV format
 * Copyright (c) 2002 Fabrice Bellard.
 *
 * This file is part of FFmpeg.
 *
 * FFmpeg is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * FFmpeg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FFmpeg; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef FFMPEG_DV_H
#define FFMPEG_DV_H

#include "avformat.h"
#include "fifo.h"

typedef struct DVDemuxContext DVDemuxContext;
DVDemuxContext* dv_init_demux(AVFormatContext* s);
int dv_get_packet(DVDemuxContext*, AVPacket *);
int dv_produce_packet(DVDemuxContext*, AVPacket*, uint8_t*, int);
void dv_offset_reset(DVDemuxContext *c, int64_t frame_offset);

typedef struct DVMuxContext DVMuxContext;
DVMuxContext* dv_init_mux(AVFormatContext* s);
int dv_assemble_frame(DVMuxContext *c, AVStream*, const uint8_t*, int, uint8_t**);
void dv_delete_mux(DVMuxContext*);

/* DJM - additional public exports for use by fastencode */

struct DVprofile;
const struct DVprofile* dv_get_codec_profile(AVCodecContext*);

#ifndef DV_MAX_FRAME_SIZE
#define DV_MAX_FRAME_SIZE 576000
#endif

struct DVMuxContext {
    const struct DVprofile* sys; /* Current DV profile. E.g.: 525/60, 625/50 */
    int         n_ast;        /* Number of stereo audio streams (up to 4) */
    AVStream   *ast[4];       /* Stereo audio streams */
    AVFifoBuffer  audio_data[4]; /* Fifo for storing excessive amounts of PCM */

    int         frames;       /* Number of a current frame */
    time_t      start_time;   /* Start time of recording */
    int         has_audio;    /* frame under contruction has audio */
    int         has_video;    /* frame under contruction has video */
    uint8_t     frame_buf[DV_MAX_FRAME_SIZE]; /* frame under contruction */
};

void dv_inject_metadata(DVMuxContext *c, uint8_t* frame);
void dv_inject_audio_raw(DVMuxContext *c, int channel, uint8_t* frame_ptr, uint8_t *data);

#endif /* FFMPEG_DV_H */
